( cd opt/kde/share/doc/HTML/da/amarok ; rm -rf common )
( cd opt/kde/share/doc/HTML/da/amarok ; ln -sf /opt/kde/share/doc/HTML/da/common common )
( cd opt/kde/share/doc/HTML/de/amarok ; rm -rf common )
( cd opt/kde/share/doc/HTML/de/amarok ; ln -sf /opt/kde/share/doc/HTML/de/common common )
( cd opt/kde/share/doc/HTML/es/amarok ; rm -rf common )
( cd opt/kde/share/doc/HTML/es/amarok ; ln -sf /opt/kde/share/doc/HTML/es/common common )
( cd opt/kde/share/doc/HTML/et/amarok ; rm -rf common )
( cd opt/kde/share/doc/HTML/et/amarok ; ln -sf /opt/kde/share/doc/HTML/et/common common )
( cd opt/kde/share/doc/HTML/fr/amarok ; rm -rf common )
( cd opt/kde/share/doc/HTML/fr/amarok ; ln -sf /opt/kde/share/doc/HTML/fr/common common )
( cd opt/kde/share/doc/HTML/it/amarok ; rm -rf common )
( cd opt/kde/share/doc/HTML/it/amarok ; ln -sf /opt/kde/share/doc/HTML/it/common common )
( cd opt/kde/share/doc/HTML/nl/amarok ; rm -rf common )
( cd opt/kde/share/doc/HTML/nl/amarok ; ln -sf /opt/kde/share/doc/HTML/nl/common common )
( cd opt/kde/share/doc/HTML/pl/amarok ; rm -rf common )
( cd opt/kde/share/doc/HTML/pl/amarok ; ln -sf /opt/kde/share/doc/HTML/pl/common common )
( cd opt/kde/share/doc/HTML/pt/amarok ; rm -rf common )
( cd opt/kde/share/doc/HTML/pt/amarok ; ln -sf /opt/kde/share/doc/HTML/pt/common common )
( cd opt/kde/share/doc/HTML/sv/amarok ; rm -rf common )
( cd opt/kde/share/doc/HTML/sv/amarok ; ln -sf /opt/kde/share/doc/HTML/sv/common common )
( cd opt/kde/share/doc/HTML/pt_BR/amarok ; rm -rf common )
( cd opt/kde/share/doc/HTML/pt_BR/amarok ; ln -sf /opt/kde/share/doc/HTML/pt_BR/common common )
( cd opt/kde/share/doc/HTML/en/amarok ; rm -rf common )
( cd opt/kde/share/doc/HTML/en/amarok ; ln -sf /opt/kde/share/doc/HTML/en/common common )
( cd opt/kde/lib64 ; rm -rf libamarok.so.0 )
( cd opt/kde/lib64 ; ln -sf libamarok.so.0.0.0 libamarok.so.0 )
( cd opt/kde/lib64 ; rm -rf libamarok.so )
( cd opt/kde/lib64 ; ln -sf libamarok.so.0.0.0 libamarok.so )
( cd opt/kde/lib64/ruby_lib ; rm -rf libhttp11.so.0 )
( cd opt/kde/lib64/ruby_lib ; ln -sf libhttp11.so.0.0.0 libhttp11.so.0 )
( cd opt/kde/lib64/ruby_lib ; rm -rf libhttp11.so )
( cd opt/kde/lib64/ruby_lib ; ln -sf libhttp11.so.0.0.0 libhttp11.so )
