body { margin: 4px; font-size: AMAROK_FONTSIZEpx; color: WindowText; background-color: Window; background-image: url( ./images/background.png ); background-repeat: no-repeat; background-attachment: fixed;}
a { font-size: AMAROK_FONTSIZEpx; color: WindowText; }
h1 { font-size: AMAROK_FONTSIZEpx; background-color: ButtonFace; color: ButtonText; font-weight: 900; border-top: 1px solid black; }
h2 { font-size: AMAROK_FONTSIZEpx; background-color: ButtonFace; color: ButtonText; font-weight: 900; border-top: 1px solid #222222; }
h3 { font-size: AMAROK_FONTSIZEpx; background-color: ButtonFace; color: ButtonText; font-weight: 900; border-top: 1px solid #444444; }
h4 { font-size: AMAROK_FONTSIZEpx; background-color: ButtonFace; color: ButtonText; font-weight: 900; border-top: 1px solid #666666; }
h5 { font-size: AMAROK_FONTSIZEpx; background-color: ButtonFace; color: ButtonText; font-weight: 900; border-top: 1px solid #888888; }
h6 { font-size: AMAROK_FONTSIZEpx; background-color: ButtonFace; color: ButtonText; font-weight: 900; border-top: 1px solid #AAAAAA; }
.button { margin: 2px; padding: 2px; border: 1px solid ButtonShadow; background-color: Window; color: WindowText; width: 100%; }
.button:hover { color: ButtonText; background-color: ButtonFace; }
.song a { display: block; padding: 1px 2px; }
.song a:hover { color: HighlightText; background-color: Highlight; }
.song-title { font-weight: bold; }
.song-score { font-variant: italic; }
.song-artist { }
.song-album { }
.song-time { }
.song-place { font-size: 12px; font-weight: 900; }
.warning { font-size: AMAROK_FONTSIZEpx; color: InfoText; background-color: InfoBackground; font-weight: bold; padding: none; margin-bottom: 1em; }
.box { border: solid ButtonShadow 1px; text-align: left; margin-bottom: AMAROK_FONTSIZEpx; }
.box a { text-decoration: none; }
.box:hover { border: solid ButtonShadow 1px; }
.box-header { color: ButtonText; background-color: ButtonFace; font-size: AMAROK_FONTSIZEpx; font-weight: bold; padding: 1px 0.5em; border-bottom: 1px solid ButtonShadow; }
.box-header:hover {}
.box-header-nav { color: ButtonText; background-color: ButtonFace; font-size: AMAROK_FONTSIZEpx; font-weight: bold; padding: 1px 0.5em; border-bottom: 1px solid ButtonShadow; text-align: right }
.box-body { background-image: url( ./images/transparency.png ); background-color: transparent; margin: 1px; }
.box-body:hover {}
.box-row {}
.box-row:hover {}
.box-row-alt {}
.box-row-alt:hover {}
.album-header {}
.album-header:hover { color: HighlightText; background-color: Highlight; cursor: pointer; }
.album-header:hover a { color: HighlightText; background-color: Highlight; cursor: pointer; }
.album-body { background-color: transparent; border-bottom: 1px solid ButtonShadow; border-top: none; }
.album-title { font-weight: bold; }
.album-info { float:right; padding-right:4px; font-size: AMAROK_FONTSIZEpx; }
.album-year { }
.album-song a { display: block; padding: 1px 2px; font-weight: normal; text-decoration: none; }
.album-song a:hover { color: HighlightText; background-color: Highlight; }
.album-song-trackno { text-align: right; width: 2em; display: block; float: left; font-weight: bold; }
.album-song-title { }
.album-song-time { }
.album-length { float:right; padding-right:4px; font-size: AMAROK_FONTSIZEpx; clear:right; }
.disc-separator { border: none; border-bottom: 1px dotted WindowText; padding-left: 2em; }
.disc-separator a { display: block; padding: 1px 2px; font-weight: 900; text-decoration: none; }
.disc-separator a:hover { color: HighlightText; background-color: Highlight; }
.statsBox { border-left: solid Highlight 1px; }
.sbtext { padding: 0px 4px; font-size: AMAROK_FONTSIZEpx; text-align: right; }
.sbouter { width: 52px; height: 10px; background-color: ThreeDFace; border: solid ThreeDShadow 1px; }
.sbinner { height: 8px; background-color: Highlight; border: solid ThreeDFace 1px; }
.ratingBox { padding: 0px 4px; }
.ratingStar { height: 1.1em; }
.default { font-size: AMAROK_FONTSIZEpx }


#favorites_box { }
#favorites_box-header { }
#favorites_box-header-title { }
#favorites_box-body { }
#newest_box { }
#newest_box-header { }
#newest_box-header-title { }
#newest_box-body { }
#least_box { }
#least_box-header { }
#least_box-header-title { }
#least_box-body { }

#current_box { }
#current_box-header { }
#current_box-header-songname { }
#current_box-header-separator { }
#current_box-header-artist { }
#current_box-header-album { font-weight: normal; }
#current_box-table { }
#current_box-largecover-td { text-align: left; width: 100px; padding: 2px; }
#current_box-largecover-a { }
#current_box-largecover-image { }
#current_box-information-td { text-align: right; font-size: AMAROK_FONTSIZEpx; }
#musicbrainz-a { }
#musicbrainz-image { width: 38px; height: 22px; }
#musicbrainz-div { }
#suggested_box { }
#suggested_box-header { }
#suggested_box-header-title { }
#T_SS { }
#favoritesby_box { }
#favoritesby-header { }
#favoritesby_box-header-title { }
#T_FT { }
#albums_box { }
#albums_box-header { }
#albums_box-header-title { }
#albums_box-body { }

#lyrics_box { }
#lyrics_box-header { }
#lyrics_box-header-title { }
#lyrics_box-body { }
