body {
	font-size:10pt;
	background-color: #9cb2cd;
	color: white;
	margin-left: 8px;
	margin-right: 8px;
}

.topnavi {
	background: url("controlbackground.png") no-repeat #E1E1E1;
}

/*
.topspace {
	height: 25px;
}
*/

div.topfixed {
	z-index: 10;
	position: fixed;
	top: 0px;
	left: 0px;
	right: 0px;
	background-color: #9cb2cd;
/* 	margin-right: 8px;*/
}

table { 
	border: 1px white; 
}   

.tracks { 
	z-index: 0;
	position: relative;
	top: 8px;
	background-color:#596082;
	border: 1px solid black 
}

table.trackheader {
	background-color:#596082;
	border: 1px solid black 
	visibility: hidden;
	display: none;
}


tr.tr_one {
	background-color: red;
/*	visibility: collapse; */
	background-color: #394062;
/*	display: none; */
}

tr.tr_two {
	background-color: red;
/*	display: none; */
/*	visibility: collapse; */
	background-color: #202050;
}

tr#trackheader {
/*	z-index: 10; */
/*	top: 30px; */
	visibility: visible;
	display: table-row;
	font-weight: bold;
	background-color:#596082;
/*	background-color: red; */
}

div.trackshmove {
	height: 0.6em;	
}

dd.tr_one { 
	background-color: #394062;
}

dd.tr_two { 	
	background-color: #202050;
}

.nowplaying { 
	border: 1pt solid red; 
}

th { 
	color: dark-grey; 
}

td {
	font-family: Geneva, Arial, Helvetica, Verdana, sans-serif;
	font-size: 10pt;
	color: #ffffff;
}

td.countdown, td#countdown {
	color: #000000;
}

.window {
	background-color: #d9d9d9; 
	border: 1px solid black;
	padding:15px;
}

a.track:link { color: white }
a.track:visited { color: white }
a.track:active { color: white }
a.track:hover { color: white }                    

a:link { 
	color: #074876; 
	border: 0px; 
	text-decoration: none; 
}
a:visited { 
	color: #074876; 
	text-decoration: none; 
}
a:active { 
	color: #074876; 	
	text-decoration: none; 
} 

a#amaroklink:hover { 
	text-decoration: none; 
	text-decoration: none; 
	background-color: #EEEEEE;
        color: white; 
}	

table.volume {
	border: 0px solid #BBBBBB;
	font-size: 16px; 
	font-family: fixed;
}

td.volumeset {
	border: 1pt solid #BBBBBB; 
	background-color: #749ce8;
	height: 15px;
	text-align: center;
	width: 15px;
}

td.volumeunset {
	border-top: 1pt solid #FFFFFF; 
	border-bottom: 1pt solid #888888; 
	border-left: 1pt solid #BBBBBB; 
	border-right: 1pt solid #AAAAAA; 
	height: 15px;
	text-align: center;
	width: 15px;
}

img {
	border: 0;
}
