# Copyright (c) 2005 Zed A. Shaw 
# You can redistribute it and/or modify it under the same terms as Ruby.
#
# Additional work donated by contributors.  See http://mongrel.rubyforge.org/attributions.html 
# for more information.

require 'rubygems'
require 'gem_plugin'

# file is just a stub that makes sure the mongrel_plugins gem is loaded and ready



